package kafka

var (
	Clouds         = []string{"aws", "azure", "gcp"}
	Availabilities = []string{"single-zone", "multi-zone"}
	Types          = []string{"basic", "standard", "enterprise", "dedicated"}
)
